import { stat } from 'fs/promises';
import { resolve } from 'path';
import { Argv } from 'yargs';
import {Stats} from 'fs';

export const description = 'Generate documentation from typedoc.json' as const;

interface GenerateArgs {
	typedocJsonPath: string;
}

const ensureTypedocJson = async (typedocPath = resolve(process.cwd(), 'typedoc.json')) => {
	const trueTypedocPath = resolve(typedocPath);
	process.stdout.write(`In path: ${trueTypedocPath}\n`);
	process.stdout.write('Does the file exist? ');
	let statResult: Stats;
	try {
		statResult = await stat(trueTypedocPath);
	} catch (errRaw) {
		if (errRaw.code === 'ENOENT') {
			process.stdout.write('no\n');
			process.stderr.write('Could not find typedoc.json\n');
			throw new Error('Could not find typedoc.json');
		}
	}
	if (statResult.isDirectory()) {
		process.stdout.write('no\n');
		process.stderr.write('typedoc.json is a directory\n');
		throw new Error('typedoc.json is a directory');
	}
	process.stdout.write('yes\n');
};

const generateTypedocData = async (cwd = process.cwd()) => {
	process.stdout.write('Generating typedoc data...\n');
	const outPath = resolve(__dirname, '..', '..', '..', '.amanuensis', 'data.json');
	const typedocBinPath = resolve(__dirname, '..', '..', '..', 'node_modules', '.bin', 'typedoc');
	const { execa } = await import('execa');
	await execa(typedocBinPath, ['--json', outPath])
		.pipeStdout(process.stdout)
		.pipeStderr(process.stderr)
	process.stdout.write('done\n');
};

export const builder = (yargs: Argv) => yargs
	.option('typedocJsonPath', {
		type: 'string',
		default: 'typedoc.json',
		alias: 't',
	});

const generate = async (args: GenerateArgs) => {
	try {
		await ensureTypedocJson(args.typedocJsonPath);
	} catch {
		return -1;
	}

	try {
		await generateTypedocData();
	} catch {
		return -2;
	}

	return 0;
};

export default generate;
