#!/usr/bin/env node

import { hideBin } from 'yargs/helpers';
import yargs from 'yargs';

import * as serve from './commands/serve';
import * as generate from './commands/generate';

const COMMANDS = {
	serve,
	generate,
};

type CommandName = keyof typeof COMMANDS;

const main = async (args: string[]) => {
	const yargsBuilder = Object.entries(COMMANDS).reduce(
		(theYargs, [name, command]) => {
			return theYargs.command(
				name,
				command.description ?? '',
				command.builder ?? (yargs => yargs),
			);
		},
		yargs
			.scriptName('amanuensis'),
	);

	const { _: commandNamesRaw, ...etcArgs } = await yargsBuilder.parse(args);

	const [commandName, ...subcommands] = commandNamesRaw as [CommandName, ...string[]];
	if (typeof commandName === 'undefined') {
		yargsBuilder.showHelp();
		return;
	}

	const { [commandName]: commandDef } = COMMANDS;
	if (typeof commandDef === 'undefined') {
		process.stderr.write(`Unknown command: ${commandName}\n`);
		yargsBuilder.showHelp();
		process.exit(-1);
		return;
	}

	const { default: handler } = commandDef;

	try {
		const returnCode = await handler({
			...etcArgs,
			_: subcommands,
		});
		process.exit(returnCode);
	} catch {
		process.exit(-1);
	}
};

void main(hideBin(process.argv));
